#!/bin/bash
# $Id: dar-backup.sh,v 1.18 2011/08/09 12:06:07 rbe Exp $
# Needs curl, dar and mutt
#
# Backup Job
set -u
set -a

KEEP_BACKUPS=4
LOGFILE="/var/log/darbackup.status"
DATE=`date -I`
DARTMPDIR=""
SLICE="-s 100M"

DAR_STATUS_PATTERN_OK='0|11'
DAR_STATUS_PATTERN_WARNING='#'


if [ -e /etc/dar-backup/dar-backup.conf ]; then
 . /etc/dar-backup/dar-backup.conf
 if [ -e /etc/dar-backup/dar-exclude.conf ]; then
  . /etc/dar-backup/dar-exclude.conf
 fi
elif [ -e /etc/backup-full.conf ]; then
 . /etc/backup-full.conf
else
 echo "No config file found for $0" >2
 exit 1
fi

if [ "x$CURLFS"!="x" ]; then
  if ! ( df -P | awk '$1 ~ /curlftpfs/ { print $6 }' | grep $CURLFS > /dev/null ); then
    pgrep -x curlftpfs && killall curlftpfs && sleep 5
    pgrep -x curlftpfs && killall -9 curlftpfs && sleep 5
    curlftpfs -o user=$FTPUSER:$FTPPASS $FTPSERVER $CURLFS || exit $?
  fi
fi

(
# Make sure FTPSERVER has a trailing / and starts with ftp://
if echo $FTPSERVER | egrep -q -v '^ftp://' > /dev/null ; then
  FTPSERVER="ftp://$FTPSERVER"
fi
if echo $FTPSERVER | egrep -q -v '/$' > /dev/null ; then
  FTPSERVER="$FTPSERVER/"
fi
# Make sure BACKUPDIR has no leading or trailing /
BACKUPDIR=`echo $BACKUPDIR | sed 's/^\///; s/\/$//'`

#cd /$BACKUPDIR || exit 1
if ! cd /$BACKUPDIR ; then
     # second try
     umount $CURLFS
     sleep 2
     pgrep -x curlftpfs && killall curlftpfs && sleep 5
     pgrep -x curlftpfs && killall -9 curlftpfs && sleep 5
     curlftpfs -o user=$FTPUSER:$FTPPASS $FTPSERVER $CURLFS || exit $?
     sleep 2
     cd /$BACKUPDIR || exit $?
fi

mkdir  $DATE || exit $?
cd /$BACKUPDIR/$DATE || exit 1

find . -maxdepth 1 -name "$HOSTID-*" -mtime +1 | xargs -r rm
[ $# -ge 1 ] && [ "$1" = "--no-slice" ] && SLICE=""

if [ -z "$DARTMPDIR" ] ; then
   dar -c /$BACKUPDIR/$DATE/$HOSTID-$DATE \
       -R / $SLICE \
       $DAREXCLUDE \
       -y \
       -G /$BACKUPDIR/$DATE/$HOSTID-$DATE-catalog \
       -D \
       -aa \
       -Q
else
  mkdir -p $DARTMPDIR || exit $?
  rm -f $DARTMPDIR/*.dar
  cd $DARTMPDIR || exit $?
  dar -c $DARTMPDIR/$HOSTID-$DATE \
    -R / $SLICE \
    $DAREXCLUDE \
    -P ${DARTMPDIR#/} \
    -y \
    -G /$BACKUPDIR/$DATE/$HOSTID-$DATE-catalog \
    -D \
    -aa \
    -Q \
    -E "/usr/bin/_dar-copy-slice %b %n"
  cd - >/dev/null || exit $?
fi
status=$?

echo "dar-exit-status: $status"
echo $status >/$BACKUPDIR/$DATE/status
[ -s "$DARTMPDIR"/darbackup-$DATE.try ] || rm -f "$DARTMPDIR"/darbackup-$DATE.try
sync


cd /$BACKUPDIR || exit 1
lastgoodstatedir=`egrep -l "$DAR_STATUS_PATTERN_OK"  [2-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]/status 2>/dev/null | tail -1 | cut -d'/' -f1`

ls -d [2-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9] |
head -n -$KEEP_BACKUPS | 
while read d; do
  [ "$d" = "$lastgoodstatedir" ] && continue
  rm -rf "$d"
done
cp /etc/motd .

verify=false
[ $# -ge 1 ] && [ "$1" = "--verify" ] && verify=true
if $verify && [ -n "$lastgoodstatedir" ]; then
  dar -t "$lastgoodstatedir/$HOSTID-$lastgoodstatedir"
  stv=$?
  [ $stv -ne 0 ] && echo $stv >"$lastgoodstatedir/status"
fi

dar_status_explain () {

        if  [ $# -eq 0 ] || [ -z "$1"  ] ; then
                echo 'bad'
        elif  echo "$1" | egrep -q "^($DAR_STATUS_PATTERN_OK)$"  ; then
                echo 'good'
        elif  echo "$1" | egrep -q "^($DAR_STATUS_PATTERN_WARNING)$"  ; then
                echo 'dont know'
        #elif [ -f $sliceerror ] ; then
        #       echo missing slice: `cat $sliceerror`
        else
                echo 'bad'
        fi
}

# logfile auf system-part. schreiben, aktueller status aller vorhandenen backups
echo -e "VERSION\t1" >$LOGFILE.new
for d in `ls -d [2-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]`; do
        st=`cat $d/status 2>/dev/null`
        [ -z "$st" ] && st='-'
        echo -e "`du -sh $d`\t$st\t`dar_status_explain $st`\t`date -r $d/status 2>/dev/null`"
done >>$LOGFILE.new

[ `wc -l $LOGFILE.new | cut -d' ' -f1` -gt 1 ] && mv -b $LOGFILE.new $LOGFILE

) 2>&1 | mutt -s "Backup $HOSTID $DATE" $BACKUPMAIL

